# encoding: utf8
from __future__ import unicode_literals
import six
from decimal import Decimal


@six.python_2_unicode_compatible
class Currency(object):
    """
    Represents a currency (unit of account).
    """

    def __init__(self, code, numeric_code, exponent=2, name=None, divisor=None, prefix=None, suffix=None):
        self.code = code
        self.numeric_code = numeric_code
        self.name = name
        self.prefix = prefix or ""
        self.suffix = suffix or ""
        # If they supplied no formatting info, use the currency code
        if not (self.prefix or self.suffix):
            self.suffix = " %s" % self.code
        # Either divisor or exponent must be provided, but not both
        if divisor is None:
            if exponent is None:
                raise ValueError("You must provide a value for divisor or exponent")
            else:
                self.exponent = exponent
                self.divisor = 10 ** self.exponent
        else:
            if exponent is None:
                self.exponent = None
                self.divisor = divisor
            else:
                raise ValueError("You cannot provide a value for both divisor and exponent")

    def __eq__(self, other):
        if not isinstance(other, Currency):
            return False
        return self.code == other.code

    def __ne__(self, other):
        return not (self == other)

    def __str__(self):
        return self.code

    def __repr__(self):
        return "<Currency %s (%s)>" % (self.code, self.name or "no name")

    def major_to_minor(self, value, force_round=False):
        """
        Converts an integer/long or Decimal value in major units
        (e.g. dollars not cents) into an integer value in minor units.

        Will error if the amount cannot be represented as an integer
        number of minor units (e.g. $3.453), unless force_round
        is passed, in which case the value will be rounded half-down.
        """
        # Don't allow imprecise types
        if not isinstance(value, six.integer_types + (Decimal, )):
            raise ValueError("The value passed in must be either an integer, a long or a decimal.")
        # Do the maths!
        minor_value = value * self.divisor
        if force_round:
            minor_value = int(round(minor_value))
        if (minor_value != int(minor_value)):
            raise ValueError(
                "Cannot convert major amount %r to minor amount; would result in fractional amount of minor unit"
                % value
            )
        return int(minor_value)

    def minor_to_major(self, value):
        """
        Converts an integer/long number of minor units (e.g. cents not dollars)
        to a Decimal value of major units.

        Note that this should not be used for further calculation (work with
        the Amount class for that), nor for display purposes (use the format functions)
        """

        # Don't allow imprecise types
        if not isinstance(value, six.integer_types):
            raise ValueError("The value passed in must be either an integer or a long")
        # Simple maths really.
        return Decimal(value) / self.divisor

    def format(self, value):
        """
        Formats a value (in the minor unit as an integer) as a string,
        with the local prefix/suffix.
        """
        return "%s%s%s" % (
            self.prefix,
            self.format_decimal(value),
            self.suffix,
        )

    def format_decimal(self, value):
        """
        Formats a value (in the minor unit as an integer) as a string,
        with no local prefix/suffix.  Can be cast into a Decimal value as needed.
        """
        major_value = self.minor_to_major(value)

        if self.exponent is not None:
            format_str = "{:.{precision}f}"
            format_kwargs = dict(precision=self.exponent)
        else:
            # A custom `divisor` can produce arbitrary precision major representations (think divisor=13)
            format_str = "{:f}"
            format_kwargs = {}

        return format_str.format(major_value, **format_kwargs)


currencies = {
    "AED": Currency("AED", "784", 2, 'UAE Dirham'),
    "AFN": Currency("AFN", "971", 2, 'Afghani'),
    "ALL": Currency("ALL", "008", 2, 'Lek'),
    "AMD": Currency("AMD", "051", 2, 'Armenian Dram'),
    "ANG": Currency("ANG", "532", 2, 'Netherlands Antillean Guilder'),
    "AOA": Currency("AOA", "973", 2, 'Kwanza'),
    "ARS": Currency("ARS", "032", 2, 'Argentine Peso'),
    "AUD": Currency("AUD", "036", 2, 'Australian Dollar'),
    "AWG": Currency("AWG", "533", 2, 'Aruban Florin'),
    "AZN": Currency("AZN", "944", 2, 'Azerbaijanian Manat'),
    "BAM": Currency("BAM", "977", 2, 'Convertible Mark'),
    "BBD": Currency("BBD", "052", 2, 'Barbados Dollar'),
    "BDT": Currency("BDT", "050", 2, 'Taka'),
    "BGN": Currency("BGN", "975", 2, 'Bulgarian Lev'),
    "BHD": Currency("BHD", "048", 3, 'Bahraini Dinar'),
    "BIF": Currency("BIF", "108", 0, 'Burundi Franc'),
    "BMD": Currency("BMD", "060", 2, 'Bermudian Dollar'),
    "BND": Currency("BND", "096", 2, 'Brunei Dollar'),
    "BOB": Currency("BOB", "068", 2, 'Boliviano'),
    "BOV": Currency("BOV", "984", 2, 'Mvdol'),
    "BRL": Currency("BRL", "986", 2, 'Brazilian Real'),
    "BSD": Currency("BSD", "044", 2, 'Bahamian Dollar'),
    "BTN": Currency("BTN", "064", 2, 'Ngultrum'),
    "BWP": Currency("BWP", "072", 2, 'Pula'),
    "BYR": Currency("BYR", "974", 0, 'Belarussian Ruble'),
    "BZD": Currency("BZD", "084", 2, 'Belize Dollar'),
    "CAD": Currency("CAD", "124", 2, 'Canadian Dollar'),
    "CDF": Currency("CDF", "976", 2, 'Congolese Franc'),
    "CHE": Currency("CHE", "947", 2, 'WIR Euro'),
    "CHF": Currency("CHF", "756", 2, 'Swiss Franc'),
    "CHW": Currency("CHW", "948", 2, 'WIR Franc'),
    "CLF": Currency("CLF", "990", 0, 'Unidad de Fomento'),
    "CLP": Currency("CLP", "152", 0, 'Chilean Peso'),
    "CNY": Currency("CNY", "156", 2, 'Yuan Renminbi'),
    "COP": Currency("COP", "170", 2, 'Colombian Peso'),
    "COU": Currency("COU", "970", 2, 'Unidad de Valor Real'),
    "CRC": Currency("CRC", "188", 2, 'Costa Rican Colon'),
    "CUC": Currency("CUC", "931", 2, 'Peso Convertible'),
    "CUP": Currency("CUP", "192", 2, 'Cuban Peso'),
    "CVE": Currency("CVE", "132", 2, 'Cape Verde Escudo'),
    "CZK": Currency("CZK", "203", 2, 'Czech Koruna'),
    "DJF": Currency("DJF", "262", 0, 'Djibouti Franc'),
    "DKK": Currency("DKK", "208", 2, 'Danish Krone'),
    "DOP": Currency("DOP", "214", 2, 'Dominican Peso'),
    "DZD": Currency("DZD", "012", 2, 'Algerian Dinar'),
    "EGP": Currency("EGP", "818", 2, 'Egyptian Pound'),
    "ERN": Currency("ERN", "232", 2, 'Nakfa'),
    "ETB": Currency("ETB", "230", 2, 'Ethiopian Birr'),
    "EUR": Currency("EUR", "978", 2, 'Euro'),
    "FJD": Currency("FJD", "242", 2, 'Fiji Dollar'),
    "FKP": Currency("FKP", "238", 2, 'Falkland Islands Pound'),
    "GBP": Currency("GBP", "826", 2, 'Pound Sterling'),
    "GEL": Currency("GEL", "981", 2, 'Lari'),
    "GHS": Currency("GHS", "936", 2, 'Ghana Cedi'),
    "GIP": Currency("GIP", "292", 2, 'Gibraltar Pound'),
    "GMD": Currency("GMD", "270", 2, 'Dalasi'),
    "GNF": Currency("GNF", "324", 0, 'Guinea Franc'),
    "GTQ": Currency("GTQ", "320", 2, 'Quetzal'),
    "GYD": Currency("GYD", "328", 2, 'Guyana Dollar'),
    "HKD": Currency("HKD", "344", 2, 'Hong Kong Dollar'),
    "HNL": Currency("HNL", "340", 2, 'Lempira'),
    "HRK": Currency("HRK", "191", 2, 'Croatian Kuna'),
    "HTG": Currency("HTG", "332", 2, 'Gourde'),
    "HUF": Currency("HUF", "348", 2, 'Forint'),
    "IDR": Currency("IDR", "360", 2, 'Rupiah'),
    "ILS": Currency("ILS", "376", 2, 'New Israeli Sheqel'),
    "INR": Currency("INR", "356", 2, 'Indian Rupee'),
    "IQD": Currency("IQD", "368", 3, 'Iraqi Dinar'),
    "IRR": Currency("IRR", "364", 2, 'Iranian Rial'),
    "ISK": Currency("ISK", "352", 0, 'Iceland Krona'),
    "JMD": Currency("JMD", "388", 2, 'Jamaican Dollar'),
    "JOD": Currency("JOD", "400", 3, 'Jordanian Dinar'),
    "JPY": Currency("JPY", "392", 0, 'Yen'),
    "KES": Currency("KES", "404", 2, 'Kenyan Shilling'),
    "KGS": Currency("KGS", "417", 2, 'Som'),
    "KHR": Currency("KHR", "116", 2, 'Riel'),
    "KMF": Currency("KMF", "174", 0, 'Comoro Franc'),
    "KPW": Currency("KPW", "408", 2, 'North Korean Won'),
    "KRW": Currency("KRW", "410", 0, 'Won'),
    "KWD": Currency("KWD", "414", 3, 'Kuwaiti Dinar'),
    "KYD": Currency("KYD", "136", 2, 'Cayman Islands Dollar'),
    "KZT": Currency("KZT", "398", 2, 'Tenge'),
    "LAK": Currency("LAK", "418", 2, 'Kip'),
    "LBP": Currency("LBP", "422", 2, 'Lebanese Pound'),
    "LKR": Currency("LKR", "144", 2, 'Sri Lanka Rupee'),
    "LRD": Currency("LRD", "430", 2, 'Liberian Dollar'),
    "LSL": Currency("LSL", "426", 2, 'Loti'),
    "LTL": Currency("LTL", "440", 2, 'Lithuanian Litas'),
    "LYD": Currency("LYD", "434", 3, 'Libyan Dinar'),
    "MAD": Currency("MAD", "504", 2, 'Moroccan Dirham'),
    "MDL": Currency("MDL", "498", 2, 'Moldovan Leu'),
    "MGA": Currency("MGA", "969", None, 'Malagasy Ariary', divisor=5),
    "MKD": Currency("MKD", "807", 2, 'Denar'),
    "MMK": Currency("MMK", "104", 2, 'Kyat'),
    "MNT": Currency("MNT", "496", 2, 'Tugrik'),
    "MOP": Currency("MOP", "446", 2, 'Pataca'),
    "MRO": Currency("MRO", "478", None, 'Ouguiya', divisor=5),
    "MUR": Currency("MUR", "480", 2, 'Mauritius Rupee'),
    "MVR": Currency("MVR", "462", 2, 'Rufiyaa'),
    "MWK": Currency("MWK", "454", 2, 'Kwacha'),
    "MXN": Currency("MXN", "484", 2, 'Mexican Peso'),
    "MXV": Currency("MXV", "979", 2, 'Mexican Unidad de Inversion (UDI)'),
    "MYR": Currency("MYR", "458", 2, 'Malaysian Ringgit'),
    "MZN": Currency("MZN", "943", 2, 'Mozambique Metical'),
    "NAD": Currency("NAD", "516", 2, 'Namibia Dollar'),
    "NGN": Currency("NGN", "566", 2, 'Naira'),
    "NIO": Currency("NIO", "558", 2, 'Cordoba Oro'),
    "NOK": Currency("NOK", "578", 2, 'Norwegian Krone'),
    "NPR": Currency("NPR", "524", 2, 'Nepalese Rupee'),
    "NZD": Currency("NZD", "554", 2, 'New Zealand Dollar'),
    "OMR": Currency("OMR", "512", 3, 'Rial Omani'),
    "PAB": Currency("PAB", "590", 2, 'Balboa'),
    "PEN": Currency("PEN", "604", 2, 'Nuevo Sol'),
    "PGK": Currency("PGK", "598", 2, 'Kina'),
    "PHP": Currency("PHP", "608", 2, 'Philippine Peso'),
    "PKR": Currency("PKR", "586", 2, 'Pakistan Rupee'),
    "PLN": Currency("PLN", "985", 2, 'Zloty'),
    "PYG": Currency("PYG", "600", 0, 'Guarani'),
    "QAR": Currency("QAR", "634", 2, 'Qatari Rial'),
    "RON": Currency("RON", "946", 2, 'New Romanian Leu'),
    "RSD": Currency("RSD", "941", 2, 'Serbian Dinar'),
    "RUB": Currency("RUB", "643", 2, 'Russian Ruble'),
    "RWF": Currency("RWF", "646", 0, 'Rwanda Franc'),
    "SAR": Currency("SAR", "682", 2, 'Saudi Riyal'),
    "SBD": Currency("SBD", "090", 2, 'Solomon Islands Dollar'),
    "SCR": Currency("SCR", "690", 2, 'Seychelles Rupee'),
    "SDG": Currency("SDG", "938", 2, 'Sudanese Pound'),
    "SEK": Currency("SEK", "752", 2, 'Swedish Krona'),
    "SGD": Currency("SGD", "702", 2, 'Singapore Dollar'),
    "SHP": Currency("SHP", "654", 2, 'Saint Helena Pound'),
    "SLL": Currency("SLL", "694", 2, 'Leone'),
    "SOS": Currency("SOS", "706", 2, 'Somali Shilling'),
    "SRD": Currency("SRD", "968", 2, 'Surinam Dollar'),
    "SSP": Currency("SSP", "728", 2, 'South Sudanese Pound'),
    "STD": Currency("STD", "678", 2, 'Dobra'),
    "SVC": Currency("SVC", "222", 2, 'El Salvador Colon'),
    "SYP": Currency("SYP", "760", 2, 'Syrian Pound'),
    "SZL": Currency("SZL", "748", 2, 'Lilangeni'),
    "THB": Currency("THB", "764", 2, 'Baht'),
    "TJS": Currency("TJS", "972", 2, 'Somoni'),
    "TMT": Currency("TMT", "934", 2, 'Turkmenistan New Manat'),
    "TND": Currency("TND", "788", 3, 'Tunisian Dinar'),
    "TOP": Currency("TOP", "776", 2, 'Pa\'anga'),
    "TRY": Currency("TRY", "949", 2, 'Turkish Lira'),
    "TTD": Currency("TTD", "780", 2, 'Trinidad and Tobago Dollar'),
    "TWD": Currency("TWD", "901", 2, 'New Taiwan Dollar'),
    "TZS": Currency("TZS", "834", 2, 'Tanzanian Shilling'),
    "UAH": Currency("UAH", "980", 2, 'Hryvnia'),
    "UGX": Currency("UGX", "800", 0, 'Uganda Shilling'),
    "USD": Currency("USD", "840", 2, 'US Dollar'),
    "UYI": Currency("UYI", "940", 0, 'Uruguay Peso en Unidades Indexadas (URUIURUI)'),
    "UYU": Currency("UYU", "858", 2, 'Peso Uruguayo'),
    "UZS": Currency("UZS", "860", 2, 'Uzbekistan Sum'),
    "VEF": Currency("VEF", "937", 2, 'Bolivar'),
    "VND": Currency("VND", "704", 0, 'Dong'),
    "VUV": Currency("VUV", "548", 0, 'Vatu'),
    "WST": Currency("WST", "882", 2, 'Tala'),
    "XAF": Currency("XAF", "950", 0, 'CFA Franc BEAC'),
    "XCD": Currency("XCD", "951", 2, 'East Caribbean Dollar'),
    "XOF": Currency("XOF", "952", 0, 'CFA Franc BCEAO'),
    "XPF": Currency("XPF", "953", 0, 'CFP Franc'),
    "YER": Currency("YER", "886", 2, 'Yemeni Rial'),
    "ZAR": Currency("ZAR", "710", 2, 'Rand'),
    "ZMW": Currency("ZMW", "967", 2, 'Zambian Kwacha'),
    "ZWL": Currency("ZWL", "932", 2, 'Zimbabwe Dollar'),
    # Why not?
    "XBT": Currency("XBT", None, 8, "Bitcoin"),
}



# encoding: utf8
from __future__ import unicode_literals
import six
from decimal import Decimal
from unittest import TestCase
from ..currency import currencies, Currency
from ..amount import Amount, _ZeroAmount


class AmountTests(TestCase):

    def test_equality(self):
        self.assertEqual(
            Amount(currencies["GBP"], 132),
            Amount(currencies["GBP"], 132),
        )
        self.assertNotEqual(
            Amount(currencies["GBP"], 132),
            Amount(currencies["USD"], 132),
        )
        self.assertNotEqual(
            Amount(currencies["GBP"], 132),
            Amount(currencies["GBP"], 99),
        )
        self.assertNotEqual(
            Amount(currencies["GBP"], 132),
            object(),
        )

    def test_add(self):
        self.assertEqual(
            Amount(currencies["GBP"], 132) + Amount(currencies["GBP"], 100),
            Amount(currencies["GBP"], 232),
        )
        with self.assertRaises(ValueError):
            Amount(currencies["GBP"], 132) + Amount(currencies["USD"], 100)

    def test_subtract(self):
        self.assertEqual(
            Amount(currencies["GBP"], 132) - Amount(currencies["GBP"], 100),
            Amount(currencies["GBP"], 32),
        )
        with self.assertRaises(ValueError):
            Amount(currencies["GBP"], 132) - Amount(currencies["USD"], 100)

    def test_comparison(self):
        self.assertLess(
            Amount(currencies["GBP"], 10),
            Amount(currencies["GBP"], 20),
        )
        self.assertGreater(
            Amount(currencies["GBP"], 20),
            Amount(currencies["GBP"], 10),
        )
        self.assertLessEqual(
            Amount(currencies["GBP"], 20),
            Amount(currencies["GBP"], 20),
        )
        self.assertGreaterEqual(
            Amount(currencies["GBP"], 20),
            Amount(currencies["GBP"], 20),
        )

    def test_format(self):
        self.assertEqual(
            six.text_type(Amount(currencies["USD"], 132)),
            "1.32 USD",
        )
        self.assertEqual(
            six.text_type(Amount(currencies["USD"], -132)),
            "-1.32 USD",
        )
        self.assertEqual(
            six.text_type(Amount(
                Currency("GBP", "826", 2, 'Pound Sterling', prefix="£"),
                132,
            )),
            "£1.32",
        )

    def test_bool(self):
        self.assertTrue(Amount(currencies["USD"], 1))
        self.assertFalse(Amount(currencies["USD"], 0))
        self.assertTrue(Amount.from_code_and_major("USD", Decimal('0.01')))
        self.assertFalse(Amount.from_code_and_major("USD", Decimal('0.00')))

    def test_apply_factor(self):
        self.assertEqual(
            Amount(currencies["GBP"], 150).apply_factor(2),
            Amount(currencies["GBP"], 300),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 100).apply_factor(Decimal("1.004")),
            Amount(currencies["GBP"], 100),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 100).apply_factor(Decimal("1.005")),
            Amount(currencies["GBP"], 101),
        )
        with self.assertRaises(ValueError):
            Amount(currencies["GBP"], 100).apply_factor(1.005)

    def test_from_code_and_minor(self):
        self.assertEqual(
            Amount.from_code_and_minor("GBP", 300),
            Amount(currencies["GBP"], 300),
        )
        self.assertEqual(
            Amount.from_code_and_minor("gbp", 300),
            Amount(currencies["GBP"], 300),
        )
        with self.assertRaises(ValueError):
            Amount.from_code_and_minor("WAITWHAT", 100)

    def test_from_code_and_major(self):
        self.assertEqual(
            Amount.from_code_and_major("GBP", "3.00"),
            Amount(currencies["GBP"], 300),
        )
        self.assertEqual(
            Amount.from_code_and_major("gbp", ".10"),
            Amount(currencies["GBP"], 10),
        )
        self.assertEqual(
            Amount.from_code_and_major("GBP", 10),
            Amount(currencies["GBP"], 1000),
        )
        self.assertEqual(
            Amount.from_code_and_major("GBP", Decimal("10.01")),
            Amount(currencies["GBP"], 1001),
        )
        with self.assertRaises(ValueError):
            Amount.from_code_and_major("WAITWHAT", 100)
        with self.assertRaises(ValueError):
            Amount.from_code_and_major("GBP", "12.432")
        with self.assertRaises(ValueError):
            Amount.from_code_and_major("GBP", "aaaaaaah")

    def test_to_major_decimal(self):
        self.assertEqual(
            Amount(currencies["GBP"], 300).to_major_decimal(),
            Decimal("3.00"),
        )
        self.assertEqual(
            Amount(currencies["USD"], 3).to_major_decimal(),
            Decimal("0.03"),
        )
        self.assertEqual(
            Amount(currencies["GBP"], -425).to_major_decimal(),
            Decimal("-4.25"),
        )
        self.assertEqual(
            Amount(currencies["MRO"], 7).to_major_decimal(),
            Decimal("1.4"),  # It's written 1.2, but is 1.4 of the major unit
        )

    def test_convert_currency(self):
        self.assertEqual(
            Amount(currencies["GBP"], 300).convert_currency("USD", 1),
            Amount(currencies["USD"], 300),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).convert_currency("USD", 1.5),
            Amount(currencies["USD"], 450),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).convert_currency("CHF", 0.43215),
            Amount(currencies["CHF"], 130),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).convert_currency("EUR", Decimal("0.91")),
            Amount(currencies["EUR"], 273),
        )

    def test_integral_division(self):
        self.assertEqual(
            Amount(currencies["GBP"], 300).integral_division(1),
            Amount(currencies["GBP"], 300),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).integral_division(3),
            Amount(currencies["GBP"], 100),
        )
        with self.assertRaises(ValueError):
            Amount(currencies["GBP"], 300).integral_division(2.2)
        with self.assertRaises(ValueError):
            Amount(currencies["GBP"], 300).integral_division(301)

    def test_divide_and_round(self):
        self.assertEqual(
            Amount(currencies["GBP"], 300).divide_and_round(1),
            Amount(currencies["GBP"], 300),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).divide_and_round(3),
            Amount(currencies["GBP"], 100),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 300).divide_and_round(4.5),
            Amount(currencies["GBP"], 67),
        )
        self.assertEqual(
            Amount(currencies["GBP"], 1).divide_and_round(2),
            Amount(currencies["GBP"], 1),
        )

class ZeroAmountTests(TestCase):
    def setUp(self):
        self.nonzero = Amount(currencies["GBP"], 300)
        self.negative = Amount(currencies["GBP"], -50)

    def test_singleton(self):
        self.assertIs(Amount.ZERO, _ZeroAmount())

    def test_simple_addition(self):
        amt = Amount.ZERO + self.nonzero
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, self.nonzero.value)

    def test_simple_raddition(self):
        amt = self.nonzero + Amount.ZERO
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, self.nonzero.value)

    def test_sum(self):
        amt = sum([self.nonzero], Amount.ZERO)
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, self.nonzero.value)

    def test_subr(self):
        amt = self.nonzero - Amount.ZERO
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, self.nonzero.value)

    def test_subl(self):
        amt = Amount.ZERO - self.nonzero
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, -self.nonzero.value)

    def test_sum_with_zeroes(self):
        amt = sum([Amount.ZERO, self.nonzero, Amount.ZERO], Amount.ZERO)
        self.assertEqual(amt.currency, self.nonzero.currency)
        self.assertEqual(amt.value, self.nonzero.value)

    def test_comparison(self):
        self.assertGreater(self.nonzero, Amount.ZERO)
        self.assertLess(Amount.ZERO, self.nonzero)
        self.assertLess(self.negative, Amount.ZERO)
        self.assertGreater(Amount.ZERO, self.negative)
        self.assertEqual(Amount.ZERO, Amount(currencies["GBP"], 0))
        self.assertEqual(Amount(currencies["GBP"], 0), Amount.ZERO)
        self.assertGreaterEqual(Amount.ZERO, Amount(currencies["GBP"], 0))
        self.assertGreaterEqual(Amount(currencies["GBP"], 0), Amount.ZERO)
        self.assertLessEqual(Amount.ZERO, Amount(currencies["GBP"], 0))
        self.assertLessEqual(Amount(currencies["GBP"], 0), Amount.ZERO)
        self.assertNotEqual(Amount(currencies["GBP"], 100), Amount.ZERO)
        self.assertNotEqual(Amount(currencies["GBP"], 100), "foobar")
        self.assertNotEqual(Amount.ZERO, "foobar")

    def test_to_major_decimal(self):
        self.assertEqual(Amount.ZERO.to_major_decimal(), Decimal('0'))

    def test_str(self):
        try:
            six.text_type(Amount.ZERO)
        except:
            self.fail("str(Amount.ZERO) raised an exception")

    def test_repr(self):
        try:
            repr(Amount.ZERO)
        except:
            self.fail("repr(Amount.ZERO) raised an exception")

    def test_forbidden_from_code_and_minor(self):
        with self.assertRaises(NotImplementedError):
            _ZeroAmount.from_code_and_minor('USD', 100)

    def test_forbidden_from_code_and_major(self):
        with self.assertRaises(NotImplementedError):
            _ZeroAmount.from_code_and_minor('USD', Decimal('1.00'))



import six
from decimal import Decimal, ROUND_HALF_UP, InvalidOperation
from functools import total_ordering


@total_ordering
@six.python_2_unicode_compatible
class Amount(object):
    """
    An amount of a currency.
    """

    def __init__(self, currency, value):
        """
        Initialises the Amount with a Currency object and an
        integer value of its minor unit (i.e. cents for USD)
        """
        assert isinstance(value, six.integer_types)
        assert not isinstance(currency, six.string_types)
        self.currency = currency
        self.value = value

    @classmethod
    def from_code_and_minor(cls, currency_code, value):
        """
        Initialises the amount with a currency code and an integer value
        of minor units
        """
        from .currency import currencies
        try:
            return cls(currencies[currency_code.upper()], value)
        except KeyError:
            raise ValueError("Invalid currency code %s" % currency_code)

    @classmethod
    def from_code_and_major(cls, currency_code, value, force_round=False):
        """
        Initialises the amount with a currency code and a value
        in the major unit (e.g. "1.43", Decimal("1.43"), 10)
        """
        from .currency import currencies
        try:
            currency = currencies[currency_code.upper()]
            return cls(currency, currency.major_to_minor(Decimal(value), force_round=force_round))
        except KeyError:
            raise ValueError("Invalid currency code %s" % currency_code)
        except InvalidOperation:
            raise ValueError("Invalid currency value %s" % value)

    def __str__(self):
        return self.currency.format(self.value)

    def __repr__(self):
        return "<Amount %s, %s>" % (self.currency, self.value)

    def __eq__(self, other):
        if not isinstance(other, Amount):
            return False
        if other is _ZeroAmount.instance:
            return other == self
        return (self.currency == other.currency) and (self.value == other.value)

    def __ne__(self, other):
        return not (self == other)

    def __add__(self, other):
        if other is _ZeroAmount.instance:
            return Amount(self.currency, self.value)
        if self.currency != other.currency:
            raise ValueError("You cannot add amounts of different currencies (%s and %s)" % (self.currency, other.currency))
        return Amount(self.currency, self.value + other.value)

    def __sub__(self, other):
        if other is _ZeroAmount.instance:
            return Amount(self.currency, self.value)
        if self.currency != other.currency:
            raise ValueError("You cannot subtract amounts of different currencies (%s and %s)" % (self.currency, other.currency))
        return Amount(self.currency, self.value - other.value)

    def __lt__(self, other):
        if other is _ZeroAmount.instance:
            return other > self
        if self.currency != other.currency:
            raise ValueError("You cannot compare amounts of different currencies (%s and %s)" % (self.currency, other.currency))
        return self.value < other.value

    def __nonzero__(self):
        return bool(self.value)

    def __bool__(self):
        return self.__nonzero__()

    def apply_factor(self, other):
        if not isinstance(other, six.integer_types + (Decimal, )):
            raise ValueError("You can only apply an integer, long or Decimal factor to an Amount")
        return Amount(
            self.currency,
            int(Decimal(self.value * other).to_integral(ROUND_HALF_UP)),
        )

    def convert_currency(self, new_code, rate):
        """
        Converts this Amount into an Amount of another currency at the given rate

        Rate is the number of new currency for each unit of the old -
        new = old * rate. Rate can be a float or a Decimal.
        """
        if not isinstance(rate, six.integer_types + (float, Decimal)):
            raise ValueError("You can only apply an integer, long, float or Decimal factor to an Amount")
        return Amount.from_code_and_minor(
            new_code,
            int(Decimal(self.value * rate).to_integral(ROUND_HALF_UP)),
        )

    def integral_division(self, divisor):
        """
        Divides the value through by the integer provided.
        Errors if the result is not exact.
        """
        if not isinstance(divisor, six.integer_types):
            raise ValueError("You can only divide by an integer or a long.")
        new_value = self.value / float(divisor)
        if int(new_value) != new_value:
            raise ValueError("Amount not exactly divisible by provided divisor")
        return Amount(self.currency, int(new_value))

    def divide_and_round(self, divisor, mode=ROUND_HALF_UP):
        """
        Divides the value through by the divisor provided.
        Optional mode from Decimal specifies rounding behaviour.
        """
        if not isinstance(divisor, six.integer_types + (float, Decimal)):
            raise ValueError("You can only divide by an integer, long, float or Decimal")
        return Amount(
            self.currency, int((Decimal(self.value) / Decimal(divisor)).to_integral_exact(mode))
        )

    def to_major_decimal(self):
        "Returns our value as a Decimal of major units"
        return self.currency.minor_to_major(self.value)


@six.python_2_unicode_compatible
class _ZeroAmount(Amount):
    """
    Amount singleton that doesn't have any currency.

    It can be used as the start of a sum() operation when you don't know the
    currency involved (yet).
    """
    instance = None

    def __init__(self):
        super(_ZeroAmount, self).__init__(None, 0)

    def __new__(cls, *args, **kwargs):
        """
        Force a singleton instance of _ZeroAmount
        """
        if cls.instance is None:
            cls.instance = super(_ZeroAmount, cls).__new__(cls)
        return cls.instance

    def __add__(self, other):
        if other is _ZeroAmount.instance:
            return self
        return Amount(other.currency, other.value)

    def __sub__(self, other):
        if other is _ZeroAmount.instance:
            return self
        return Amount(other.currency, -other.value)

    def __eq__(self, other):
        if not isinstance(other, Amount):
            return False
        return self.value == other.value

    def __lt__(self, other):
        return self.value < other.value

    def __str__(self):
        return six.text_type(self.value)

    def __nonzero__(self):
        return False

    def __bool__(self):
        return False

    @classmethod
    def from_code_and_minor(cls, currency_code, value):
        raise NotImplementedError

    @classmethod
    def from_code_and_major(cls, currency_code, value):
        raise NotImplementedError

    def to_major_decimal(self):
        return Decimal(self.value)


Amount.ZERO = _ZeroAmount()



from setuptools import setup, find_packages
from currint import __version__

setup(
    name='currint',
    version=__version__,
    description='Integer-based, fixed precision currency calculation',
    packages=find_packages(),
    author='Andrew Godwin',
    author_email='andrew@aeracode.org',
    install_requires=[
        'six',
    ],
    test_suite='currint.tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development',
    ],
)




# encoding: utf8
from __future__ import unicode_literals
from decimal import Decimal
from unittest import TestCase
from ..currency import currencies, Currency


class CurrencyTests(TestCase):

    def test_major_to_minor(self):
        self.assertEqual(
            currencies["GBP"].major_to_minor(1),
            100,
        )
        self.assertEqual(
            currencies["GBP"].major_to_minor(Decimal("1.43")),
            143,
        )
        with self.assertRaises(ValueError):
            currencies["GBP"].major_to_minor(Decimal("1.435"))
        # Non-decimal currency
        self.assertEqual(
            currencies["MRO"].major_to_minor(Decimal(5)),
            25,
        )

    def test_minor_to_major(self):
        self.assertEqual(
            currencies["GBP"].minor_to_major(100),
            Decimal("1"),
        )
        self.assertEqual(
            currencies["GBP"].minor_to_major(143),
            Decimal("1.43"),
        )
        # Non-decimal currency
        self.assertEqual(
            currencies["MRO"].minor_to_major(26),
            Decimal("5.2"),
        )

    def test_format(self):
        self.assertEqual(
            Currency("GBP", "826", 2, 'Pound Sterling', prefix="£").format(100),
            "£1.00",
        )
        self.assertEqual(
            currencies["USD"].format(43),
            "0.43 USD",
        )
        # Non-decimal currency
        self.assertEqual(
            currencies["MRO"].format(7),
            "1.4 MRO",
        )
        # Negative value less than 0 and greater than -1 major unit
        self.assertEqual(
            currencies["USD"].format(-60),
            "-0.60 USD",
        )

        # 0 exponent currency
        self.assertEqual(
            currencies["JPY"].format(100),
            "100 JPY",
        )

    def test_format_decimal(self):
        self.assertEqual(
            currencies["GBP"].format_decimal(100),
            "1.00",
        )
        self.assertEqual(
            currencies["USD"].format_decimal(43),
            "0.43",
        )
        # Non-decimal currency
        self.assertEqual(
            currencies["MRO"].format_decimal(7),
            "1.4",
        )

        # Negative value less than 0 and greater than -1 major unit
        self.assertEqual(
            currencies["USD"].format_decimal(-60),
            "-0.60",
        )

        # 0 exponent currency
        self.assertEqual(
            currencies["JPY"].format_decimal(100),
            "100",
        )

    def test_equality(self):
        self.assertEqual(
            currencies['GBP'],
            currencies['GBP']
        )
        self.assertNotEqual(
            currencies['GBP'],
            currencies['USD']
        )
        self.assertNotEqual(
            currencies['GBP'],
            object(),
        )
